/*
 * Copyright (c) 2018 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Notwithstanding the foregoing, you may not use, copy, modify, merge, publish,
 * distribute, sublicense, create a derivative work, and/or sell copies of the
 * Software in any work that is designed, intended, or marketed for pedagogical or
 * instructional purposes related to programming, coding, application development,
 * or information technology.  Permission for such use, copying, modification,
 * merger, publication, distribution, sublicensing, creation of derivative works,
 * or sale is expressly withheld.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package com.raywenderlich.android.fontquiz

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.graphics.Color
import android.os.Bundle
import android.support.v4.content.ContextCompat
import android.support.v4.view.ViewCompat
import android.support.v7.app.AlertDialog
import android.support.v7.app.AppCompatActivity
import android.view.View
import android.widget.Button
import android.widget.TextView
import android.widget.Toast
import kotlinx.android.synthetic.main.activity_question.*
import java.util.*

class QuestionActivity : AppCompatActivity() {

  companion object {
    const val EXTRA_NUMBER_OF_QUESTIONS = "com.raywenderlich.android.fontquiz.QuestionActivity::number-of-questions"
    const val EXTRA_CURRENT_QUESTION = "com.raywenderlich.android.fontquiz.QuestionActivity::current-question"
    const val EXTRA_CORRECT_ANSWERS = "com.raywenderlich.android.fontquiz.QuestionActivity::correct-answers"

    const val REQUEST_NEXT_QUESTION = 100

    fun buildIntent(context: Context, numberOfQuestions: Int): Intent {
      return buildIntent(context, numberOfQuestions, 0, 0)
    }

    private fun buildIntent(context: Context, numberOfQuestions: Int, currentQuestion: Int,
                            correctAnswers: Int): Intent {
      return Intent(context, QuestionActivity::class.java)
          .putExtra(EXTRA_NUMBER_OF_QUESTIONS, numberOfQuestions)
          .putExtra(EXTRA_CURRENT_QUESTION, currentQuestion)
          .putExtra(EXTRA_CORRECT_ANSWERS, correctAnswers)
    }
  }

  private val numberOfQuestions
    get() = intent.getIntExtra(EXTRA_NUMBER_OF_QUESTIONS, 0)

  private val currentQuestion
    get() = intent.getIntExtra(EXTRA_CURRENT_QUESTION, 0)

  private var correctAnswers
    get() = intent.getIntExtra(EXTRA_CORRECT_ANSWERS, 0)
    set(value) {
      intent.putExtra(EXTRA_CORRECT_ANSWERS, value)
    }

  private val buttons: ArrayList<Button>
    get() {
      return arrayListOf(firstAnswerButton, secondAnswerButton, thirdAnswerButton, fourthAnswerButton)
    }


  override fun onCreate(savedInstanceState: Bundle?) {
    super.onCreate(savedInstanceState)
    setContentView(R.layout.activity_question)

    progressTextView.text = getString(R.string.question_number, currentQuestion + 1, numberOfQuestions)

    generateQuestion()
  }

  override fun onBackPressed() {
    showExitQuestion()
  }

  override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
    // Pass the result back to previous activity.
    setResult(resultCode)
    finish()
  }

  private fun generateQuestion() {
    val random = Random()

    val familyNameList = getRandomFamilyNameList(random, buttons.size)
    val correctAnswer = random.nextInt(buttons.size)

    showFont(familyNameList[correctAnswer])

    for (i in 0..(buttons.size - 1)) {
      setupAnswer(buttons[i], familyNameList[i], familyNameList[correctAnswer])
    }
  }

  private fun getRandomFamilyNameList(random: Random, size: Int): ArrayList<String> {
    val fontFamilyNames = resources.getStringArray(R.array.family_names)
    val familyNameSet = HashSet<String>()

    while (familyNameSet.size < size) {
      // Ensure we have exactly 4 family names without duplicates.
      familyNameSet.add(fontFamilyNames[random.nextInt(fontFamilyNames.size)])
    }

    return ArrayList(familyNameSet)
  }

  private fun showFont(familyName: String) {

  }

  private fun setupAnswer(button: Button, text: String, correctAnswer: String) {
    with(button) {
      setText(text)
      tag = correctAnswer

      setOnClickListener({ view ->
        ViewCompat.setBackgroundTintList(view, ContextCompat.getColorStateList(this@QuestionActivity, R.color.colorPrimary))
        showAnswer(text == correctAnswer, correctAnswer)
      })
    }
  }

  private fun showAnswer(isCorrect: Boolean, answerFamilyName: String) {
    buttons.forEach { button ->
      button.isEnabled = false
      button.isClickable = false
    }

    loadFontFact(messageTextView, answerFamilyName)

    with(feedbackImageView) {
      if (isCorrect) {
        setImageResource(R.drawable.ic_correct)
      } else {
        setImageResource(R.drawable.ic_wrong)
      }
      visibility = View.VISIBLE
    }

    with(feedbackTextView) {
      if (isCorrect) {
        text = getString(R.string.correct)
      } else {
        text = getString(R.string.wrong)
      }
      visibility = View.VISIBLE
    }

    with(rootLayout) {
      setOnClickListener({
        if (currentQuestion == numberOfQuestions - 1) {
          // We've reached the end of quiz, so let's show the score.
          showScore(correctAnswers, numberOfQuestions)
        } else {
          showNextQuestion(isCorrect)
        }
      })

      if (isCorrect) {
        setBackgroundColor(Color.GREEN)
      } else {
        setBackgroundColor(Color.RED)
      }
    }
  }

  private fun loadFontFact(textView: TextView, familyName: String) {

  }

  private fun showNextQuestion(answerWasCorrect: Boolean) {
    if (answerWasCorrect) {
      correctAnswers++
    }

    startActivityForResult(
        QuestionActivity.buildIntent(
            this,
            numberOfQuestions,
            currentQuestion + 1,
            correctAnswers
        ),
        REQUEST_NEXT_QUESTION
    )
  }

  private fun showScore(correctAnswers: Int, numberOfQuestions: Int) {
    AlertDialog.Builder(this)
        .setTitle(getString(R.string.quiz_complete))
        .setMessage(getString(R.string.quiz_score_message, correctAnswers, numberOfQuestions))
        .setPositiveButton(R.string.ok, { _, _ ->
          setResult(Activity.RESULT_OK)
          finish()
        })
        .show()
  }

  private fun showExitQuestion() {
    AlertDialog.Builder(this)
        .setTitle(getString(R.string.exit_quiz))
        .setMessage(getString(R.string.exit_quiz_message))
        .setPositiveButton(R.string.yes, { _, _ -> finish() })
        .setNegativeButton(R.string.no, null)
        .show()
  }

  private fun showError(reason: Int) {
    Toast.makeText(
        this@QuestionActivity,
        getString(R.string.request_failed, reason),
        Toast.LENGTH_LONG
    ).show()

    progressView.visibility = View.GONE
    finish()
  }
}
